/*--------------------------------------------------------------------------*/
/* File name : send.c                                                       */
/* Author    : Yoji Suzuki                                                  */
/* Outline   : Testing of TCP/IP connection and send the message.           */
/*--------------------------------------------------------------------------*/
#include <WinSock2.h>
#include <Windows.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

typedef unsigned long int in_addr_t;

/*--------------------------------------------------------------------------*/
/* Proto-types                                                              */
/*--------------------------------------------------------------------------*/
static int fs_Connect();
static int fs_Send();
static int fs_Recv();

/*--------------------------------------------------------------------------*/
/* main() : Entry function of this program.                                 */
/*--------------------------------------------------------------------------*/
int main(argc,argv)
int argc;
char* argv[];
{
  WSADATA wsaData;
  SOCKET sock;
  in_addr_t addr;
  int iRet;
  short port;
  /* STDIN & STDOUT are changing to the binary mode. */
  if(_setmode(_fileno(stdin),O_BINARY)==-1) {
    perror("setmode()");
    return 127;
  }
  if(_setmode(_fileno(stdout),O_BINARY)==-1) {
    perror("setmode()");
    return 127;
  }
  /* Initialization of the Windows Socket version 2. */
  if(WSAStartup(MAKEWORD(2,0),&wsaData)!=0) {
    fprintf(stderr,"WSAStartup() error.(%d)\n",WSAGetLastError());
    return 127;
  }
  /* Arguments check. */
  if(argc<3) {
    fprintf(stderr,"usage:");
    fprintf(stderr,">send address port");
    WSACleanup();
    return 1;
  }
  addr=inet_addr(argv[1]);
  port=htons((short)atoi(argv[2]));
  /* Create socket */
  sock=socket(AF_INET,SOCK_STREAM,0);
  if(sock==INVALID_SOCKET) {
    fprintf(stderr,"socket() error.(%d)\n",WSAGetLastError());
    WSACleanup();
    return 127;
  }
  /* Connect */
  if(fs_Connect(sock,addr,port)!=0) {
    closesocket(sock);
    WSACleanup();
    return 2;
  }
  while(1) {
    /* Send */
    if((iRet=fs_Send(sock))<0) {
      shutdown(sock,SD_SEND);
      closesocket(sock);
      WSACleanup();
      return 3;
    } else if(iRet==0) break;
    /* Recv */
    if(fs_Recv(sock)!=0) {
      shutdown(sock,SD_SEND);
      closesocket(sock);
      WSACleanup();
      return 4;
    }
  }
  /* Termination. */
  shutdown(sock,SD_SEND);
  closesocket(sock);
  WSACleanup();
  return 0;
}

/*--------------------------------------------------------------------------*/
/* fs_Connect() : Connect to the server.                                    */
/*--------------------------------------------------------------------------*/
static int fs_Connect(sock,addr,port)
SOCKET sock;
in_addr_t addr;
short port;
{
  struct sockaddr_in inSAddr;
  fprintf(stderr,"> Connecting...");
  memset(&inSAddr,0,sizeof(inSAddr));
  inSAddr.sin_family=AF_INET;
  inSAddr.sin_port=port;
  inSAddr.sin_addr.s_addr=addr;
  if(connect(sock,(struct sockaddr*)&inSAddr,sizeof(inSAddr))<0) {
    fprintf(stderr,"connect() error.(%d)\n",WSAGetLastError());
    return -1;
  }
  fprintf(stderr,"success.\n");
  return 0;
}

/*--------------------------------------------------------------------------*/
/* fs_Send() : Sending the server the standard inputs data.                 */
/*--------------------------------------------------------------------------*/
static int fs_Send(sock)
SOCKET sock;
{
  char cMsg[1024];
  int iByte;
  int iSendBufSize;
  /* Send data */
  fprintf(stderr,"> Inputing data. (CTRL+Z to quit)\n");
  if(fread(cMsg,1,sizeof(cMsg),stdin)<=0) {
    return -1;
  }
  fprintf(stderr,"> Sending...");
  for(iSendBufSize=0;cMsg[iSendBufSize]!='\0';iSendBufSize++);
  iSendBufSize--; /* Delete '\0' */
  iByte=send(sock,cMsg,iSendBufSize,0);
  if(iByte<0) {
    fprintf(stderr,"send() error.(%d)\n",WSAGetLastError());
    return -1;
  }
  fprintf(stderr,"success. (%d bytes)\n",iByte);
  return 1;
}

/*--------------------------------------------------------------------------*/
/* fs_Recv()                                                                */
/*--------------------------------------------------------------------------*/
static int fs_Recv(sock)
SOCKET sock;
{
  char rbuf[4096];
  int iByte;
  int iRecvBufSize=(int)sizeof(rbuf)-1;
  int iLoop=1;
  /* Recv data */
  while(iLoop) {
    fprintf(stderr,"> Receiving...");
    iByte=recv(sock,rbuf,iRecvBufSize,0);
    if(iByte<0) {
      fprintf(stderr,"recv() error.(%d)\n",WSAGetLastError());
      return -1;
    }
    fprintf(stderr,"success. (%d bytes)\n",iByte);
    rbuf[iByte+1]='\0';
    puts(rbuf);
    if(iByte<iRecvBufSize) iLoop=0;
  }
  return 0;
}

/* End of send.c */
